/***************************************************************************
 *            textures.cpp
 *
 *  Thu Dec  1 17:46:50 2005
 *  Copyright  2005  Joe Venzon
 *  joe@venzon.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include "textures.h"

TEXTURES::~TEXTURES()
{
	DeleteAll();
}

void TEXTURES::DeleteAll()
{
	/*list <TEXTURE>::iterator tex;

	for( tex = textures.begin(); textures.begin() != textures.end(); tex = textures.begin() )
	{
		textures.erase( tex );
	}*/
	
	textures.clear();
}


void TEXTURES::Delete( GLuint tex_id )
{
	list <TEXTURE>::iterator tex;

	for( tex = textures.begin(); tex != textures.end(); ++tex )
	{
		if( tex->GetTexID() == tex_id )
		{
			list <TEXTURE>::iterator oldpos = tex;
			if (tex != textures.begin())
			{
				oldpos = tex--;
				textures.erase( tex );
				tex = oldpos;
			}
			else
			{
				textures.erase( tex );
				tex = textures.begin();
			}
		}
	}
}

void TEXTURES::Delete( string file )
{
	list<TEXTURE>::iterator tex;

	for( tex = textures.begin(); tex != textures.end(); ++tex )
	{
		if( tex->GetFile() == file )
		{
			list <TEXTURE>::iterator oldpos = tex;
			if (tex != textures.begin())
			{
				oldpos = tex--;
				textures.erase( tex );
				tex = oldpos;
			}
			else
			{
				textures.erase( tex );
				tex = textures.begin();
			}
		}
	}
}

GLuint TEXTURES::Load( string new_tex_file, bool mipmap, int &w, int &h )
{
	list <TEXTURE>::iterator tex;
	GLuint new_tex_id;
	bool error = false;

	for( tex = textures.begin(); tex != textures.end(); ++tex )
	{
		if( tex->GetFile() == new_tex_file )
		{
			return tex->GetTexID();
		}
	}

	new_tex_id = utility.TexLoad( new_tex_file, GL_RGBA, mipmap, w, h, false, error );

	if( error )
	{
		cout << "error loading texture file \"" << new_tex_file << "\"..." << endl;
		return 0;
	}
	else
	{
		TEXTURE new_tex;
		textures.push_front( new_tex );
		textures.begin()->Set(new_tex_file, new_tex_id, mipmap);
		return new_tex_id;
	}
}

void TEXTURES::ReloadAll()
{
	//list <TEXTURE>::iterator tex;
	GLuint tex_id;
	string file;
	int w, h;
	bool mipmap;
	bool error;

	/*TEXTURE new_tex;
	int i = 0;
	int num_textures = textures.size();
	tex = textures.begin();
	for( i = 0; i < num_textures; i++ )
	{
		file = tex->GetFile();
		mipmap = (tex)->GetMipmap();
		tex_id = utility.TexLoad( file, GL_RGBA, mipmap, w, h, false, error);
		if( error ) cout << "error encountered while reloading texture \"" << file << "\"..." << endl;

		new_tex = new TEXTURE( file, tex_id, mipmap );
		textures.push_back( new_tex );

		delete *tex;
		textures.erase( tex );
		tex = textures.begin();
	}*/
	
	for (list <TEXTURE>::iterator tex = textures.begin(); tex != textures.end(); tex++)
	{
		tex->Unload();
		file = tex->GetFile();
		mipmap = tex->GetMipmap();
		tex_id = utility.TexLoad( file, GL_RGBA, mipmap, w, h, false, error);
		if( error ) cout << "error encountered while reloading texture \"" << file << "\"..." << endl;

		tex->Set(file, tex_id, mipmap);
	}
}
